# coding: utf-8

# flake8: noqa
"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 1.11.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

# import models into model package
from curia.api.swagger_client.models.analysis import Analysis
from curia.api.swagger_client.models.analysis_job import AnalysisJob
from curia.api.swagger_client.models.analysis_job_output import AnalysisJobOutput
from curia.api.swagger_client.models.analysis_job_status import AnalysisJobStatus
from curia.api.swagger_client.models.body import Body
from curia.api.swagger_client.models.code import Code
from curia.api.swagger_client.models.cohort import Cohort
from curia.api.swagger_client.models.cohort_filter import CohortFilter
from curia.api.swagger_client.models.cohort_filter_condition import CohortFilterCondition
from curia.api.swagger_client.models.cohort_window import CohortWindow
from curia.api.swagger_client.models.create_many_analysis_job_dto import CreateManyAnalysisJobDto
from curia.api.swagger_client.models.create_many_analysis_job_output_dto import CreateManyAnalysisJobOutputDto
from curia.api.swagger_client.models.create_many_analysis_job_status_dto import CreateManyAnalysisJobStatusDto
from curia.api.swagger_client.models.create_many_code_dto import CreateManyCodeDto
from curia.api.swagger_client.models.create_many_cohort_dto import CreateManyCohortDto
from curia.api.swagger_client.models.create_many_data_query_dto import CreateManyDataQueryDto
from curia.api.swagger_client.models.create_many_dataset_column_dto import CreateManyDatasetColumnDto
from curia.api.swagger_client.models.create_many_dataset_dto import CreateManyDatasetDto
from curia.api.swagger_client.models.create_many_dataset_job_dto import CreateManyDatasetJobDto
from curia.api.swagger_client.models.create_many_feature_category_dto import CreateManyFeatureCategoryDto
from curia.api.swagger_client.models.create_many_feature_dto import CreateManyFeatureDto
from curia.api.swagger_client.models.create_many_feature_sub_category_dto import CreateManyFeatureSubCategoryDto
from curia.api.swagger_client.models.create_many_model_batch_dto import CreateManyModelBatchDto
from curia.api.swagger_client.models.create_many_model_batch_job_dto import CreateManyModelBatchJobDto
from curia.api.swagger_client.models.create_many_model_dataset_dto import CreateManyModelDatasetDto
from curia.api.swagger_client.models.create_many_model_dto import CreateManyModelDto
from curia.api.swagger_client.models.create_many_model_job_dto import CreateManyModelJobDto
from curia.api.swagger_client.models.create_many_model_job_output_dto import CreateManyModelJobOutputDto
from curia.api.swagger_client.models.create_many_organization_dto import CreateManyOrganizationDto
from curia.api.swagger_client.models.create_many_organization_feature_exclusion_dto import CreateManyOrganizationFeatureExclusionDto
from curia.api.swagger_client.models.create_many_organization_setting_dto import CreateManyOrganizationSettingDto
from curia.api.swagger_client.models.create_many_process_dto import CreateManyProcessDto
from curia.api.swagger_client.models.create_many_process_job_dto import CreateManyProcessJobDto
from curia.api.swagger_client.models.create_many_process_job_output_dto import CreateManyProcessJobOutputDto
from curia.api.swagger_client.models.create_many_process_job_status_dto import CreateManyProcessJobStatusDto
from curia.api.swagger_client.models.create_many_project_dto import CreateManyProjectDto
from curia.api.swagger_client.models.create_many_project_member_dto import CreateManyProjectMemberDto
from curia.api.swagger_client.models.create_many_user_favorite_dto import CreateManyUserFavoriteDto
from curia.api.swagger_client.models.data_query import DataQuery
from curia.api.swagger_client.models.data_table import DataTable
from curia.api.swagger_client.models.database import Database
from curia.api.swagger_client.models.dataset import Dataset
from curia.api.swagger_client.models.dataset_column import DatasetColumn
from curia.api.swagger_client.models.dataset_job import DatasetJob
from curia.api.swagger_client.models.dataset_job_status import DatasetJobStatus
from curia.api.swagger_client.models.feature import Feature
from curia.api.swagger_client.models.feature_category import FeatureCategory
from curia.api.swagger_client.models.feature_sub_category import FeatureSubCategory
from curia.api.swagger_client.models.feature_table import FeatureTable
from curia.api.swagger_client.models.geographic_counts import GeographicCounts
from curia.api.swagger_client.models.geographic_queries import GeographicQueries
from curia.api.swagger_client.models.get_many_analysis_job_output_response_dto import GetManyAnalysisJobOutputResponseDto
from curia.api.swagger_client.models.get_many_analysis_job_response_dto import GetManyAnalysisJobResponseDto
from curia.api.swagger_client.models.get_many_analysis_job_status_response_dto import GetManyAnalysisJobStatusResponseDto
from curia.api.swagger_client.models.get_many_code_response_dto import GetManyCodeResponseDto
from curia.api.swagger_client.models.get_many_cohort_response_dto import GetManyCohortResponseDto
from curia.api.swagger_client.models.get_many_data_query_response_dto import GetManyDataQueryResponseDto
from curia.api.swagger_client.models.get_many_data_table_response_dto import GetManyDataTableResponseDto
from curia.api.swagger_client.models.get_many_database_response_dto import GetManyDatabaseResponseDto
from curia.api.swagger_client.models.get_many_dataset_column_response_dto import GetManyDatasetColumnResponseDto
from curia.api.swagger_client.models.get_many_dataset_job_response_dto import GetManyDatasetJobResponseDto
from curia.api.swagger_client.models.get_many_dataset_job_status_response_dto import GetManyDatasetJobStatusResponseDto
from curia.api.swagger_client.models.get_many_dataset_response_dto import GetManyDatasetResponseDto
from curia.api.swagger_client.models.get_many_feature_category_response_dto import GetManyFeatureCategoryResponseDto
from curia.api.swagger_client.models.get_many_feature_response_dto import GetManyFeatureResponseDto
from curia.api.swagger_client.models.get_many_feature_sub_category_response_dto import GetManyFeatureSubCategoryResponseDto
from curia.api.swagger_client.models.get_many_model_batch_job_response_dto import GetManyModelBatchJobResponseDto
from curia.api.swagger_client.models.get_many_model_batch_response_dto import GetManyModelBatchResponseDto
from curia.api.swagger_client.models.get_many_model_dataset_response_dto import GetManyModelDatasetResponseDto
from curia.api.swagger_client.models.get_many_model_job_output_response_dto import GetManyModelJobOutputResponseDto
from curia.api.swagger_client.models.get_many_model_job_response_dto import GetManyModelJobResponseDto
from curia.api.swagger_client.models.get_many_model_job_status_response_dto import GetManyModelJobStatusResponseDto
from curia.api.swagger_client.models.get_many_model_response_dto import GetManyModelResponseDto
from curia.api.swagger_client.models.get_many_organization_feature_exclusion_response_dto import GetManyOrganizationFeatureExclusionResponseDto
from curia.api.swagger_client.models.get_many_organization_response_dto import GetManyOrganizationResponseDto
from curia.api.swagger_client.models.get_many_organization_setting_response_dto import GetManyOrganizationSettingResponseDto
from curia.api.swagger_client.models.get_many_process_job_output_response_dto import GetManyProcessJobOutputResponseDto
from curia.api.swagger_client.models.get_many_process_job_response_dto import GetManyProcessJobResponseDto
from curia.api.swagger_client.models.get_many_process_job_status_response_dto import GetManyProcessJobStatusResponseDto
from curia.api.swagger_client.models.get_many_process_response_dto import GetManyProcessResponseDto
from curia.api.swagger_client.models.get_many_project_member_response_dto import GetManyProjectMemberResponseDto
from curia.api.swagger_client.models.get_many_project_response_dto import GetManyProjectResponseDto
from curia.api.swagger_client.models.get_many_user_favorite_response_dto import GetManyUserFavoriteResponseDto
from curia.api.swagger_client.models.model import Model
from curia.api.swagger_client.models.model_batch import ModelBatch
from curia.api.swagger_client.models.model_batch_job import ModelBatchJob
from curia.api.swagger_client.models.model_dataset import ModelDataset
from curia.api.swagger_client.models.model_job import ModelJob
from curia.api.swagger_client.models.model_job_config import ModelJobConfig
from curia.api.swagger_client.models.model_job_event import ModelJobEvent
from curia.api.swagger_client.models.model_job_output import ModelJobOutput
from curia.api.swagger_client.models.model_job_output_feature import ModelJobOutputFeature
from curia.api.swagger_client.models.model_job_status import ModelJobStatus
from curia.api.swagger_client.models.organization import Organization
from curia.api.swagger_client.models.organization_feature_exclusion import OrganizationFeatureExclusion
from curia.api.swagger_client.models.organization_setting import OrganizationSetting
from curia.api.swagger_client.models.period_definition import PeriodDefinition
from curia.api.swagger_client.models.process import Process
from curia.api.swagger_client.models.process_job import ProcessJob
from curia.api.swagger_client.models.process_job_output import ProcessJobOutput
from curia.api.swagger_client.models.process_job_status import ProcessJobStatus
from curia.api.swagger_client.models.project import Project
from curia.api.swagger_client.models.project_member import ProjectMember
from curia.api.swagger_client.models.update_user_dto import UpdateUserDto
from curia.api.swagger_client.models.user_favorite import UserFavorite



"""
model.py
====================================
Abstract Base Class Model Class used for Risk & Impactability Models
"""
from abc import ABC
from datetime import datetime

from logzero import logger
from pandas import DataFrame
from validation_decorators import ValidateArgType
from validation_decorators.errors import raise_error
import tempfile

from curia.api.swagger_client import Model

type_validator = ValidateArgType(raise_error, logger=logger)


class _Model(ABC):

    def __init__(
            self,
            session,
            name: str,
            description: str = None,
            model_type: str = None,
            model_id: str = None,
            model_dataset_id: str = None,
            project_id: str = None,
            environment_id: str = None,
            last_train_model_dataset_id: str = None,
    ):

        self.session = session

        self.name = name
        self.description = description
        self.feature_store = 'byod'

        self.model_type = model_type
        self.model_id = model_id
        self.model_dataset_id = model_dataset_id
        self.project_id = project_id
        self.environment_id = environment_id
        self.last_train_model_dataset_id = last_train_model_dataset_id

        self._job_type = None

    def set_name(self, name: str):
        """

        Parameters
        ----------
        name: str :
            

        Returns
        -------

        """

        self.session.logger.debug(name)
        self.name = name

    def set_description(self, description: str):
        """

        Parameters
        ----------
        description: str :
            

        Returns
        -------

        """

        self.session.logger.debug(description)
        self.description = description

    def set_model_id(self, model_id: str):
        """

        Parameters
        ----------
        model_id: str :
            

        Returns
        -------

        """

        self.session.logger.debug(model_id)
        self.model_id = model_id

    def set_model_dataset_id(self, model_dataset_id: str):
        """

        Parameters
        ----------
        model_dataset_id: str :
            

        Returns
        -------

        """

        self.session.logger.debug(model_dataset_id)
        self.model_dataset_id = model_dataset_id

    def set_project_id(self, project_id: str):
        """

        Parameters
        ----------
        project_id: str :
            

        Returns
        -------

        """

        self.session.logger.debug(project_id)
        self.project_id = project_id

    def set_environment_id(self, environment_id: str):
        """

        Parameters
        ----------
        environment_id: str :
            

        Returns
        -------

        """

        self.session.logger.debug(environment_id)
        self.environment_id = environment_id

    def set_job_type(self, job_type: str):
        """

        Parameters
        ----------
        job_type: str :
            

        Returns
        -------

        """

        self.session.logger.debug(job_type)
        self._job_type = job_type

    @type_validator(data=DataFrame)
    def model_dataset_upload(self, data: DataFrame):
        """

        Parameters
        ----------
        data: DataFrame :
            

        Returns
        -------

        """

        self.session.logger.info('uploading DataFrame with shape %s', data.shape)
        if self.model_id is None:
            model: Model = self.session.api_instance.create_one_base_model_controller_model(
                self.session.api_client.Model(
                    name=self.name,
                    type=self.model_type,
                    feature_store=self.feature_store,
                    description=self.description,
                    project_id=self.project_id
                )
            )
        else:
            model = self.session.api_instance.get_one_base_model_controller_model(self.model_id)

        self.session.logger.info('created new model: %s', model)
        self.model_id = model.id

        dataset = self.session.api_instance.create_one_base_dataset_controller_dataset(
            self.session.api_client.Dataset(
                name=f'{self.model_id}-{self._job_type}-{datetime.now()}',
                description='BYOD train dataset uploaded via curia-python-sdk',
                type=self._job_type,
                file_size=int(data.memory_usage(index=False).sum()),
                dataset_results={}
            )
        )

        self.session.logger.info('dataset data: %s', dataset)
        with tempfile.NamedTemporaryFile(mode='w', delete=False) as temp:
            data.to_csv(temp, chunksize=1000)
        self.session.api_instance.dataset_controller_upload(
            temp.name,
            dataset.id
        )
        self.session.logger.info('upload successful')

        model_dataset = self.session.api_instance.create_one_base_model_dataset_controller_model_dataset(
            self.session.api_client.ModelDataset(
                dataset_id=dataset.id,
                model_id=self.model_id,
                train_model_dataset_id=self.last_train_model_dataset_id if self._job_type == 'predict' else None
            )
        )
        if self._job_type == 'train':
            self.last_train_model_dataset_id = model_dataset.id
        self.session.logger.info('created model dataset: %s', model_dataset)

        self.model_dataset_id = model_dataset.id

        return model_dataset

    def start(self):
        """Start the model job"""
        self.session.logger.info(
            'starting model job: type=%s modelDatasetId=%s environmentId=%s projectId=%s',
            self._job_type,
            self.model_dataset_id,
            self.environment_id,
            self.project_id
        )
        model_job = self.session.api_instance.create_one_base_model_job_controller_model_job(body={
            'type': self._job_type,
            'modelDatasetId': self.model_dataset_id,
            'environmentId': self.environment_id,
            'projectId': self.project_id
        })
        self.session.logger.info('model job created: %s', model_job)

        self.session.api_instance.model_job_controller_start(model_job.id, type=self._job_type)
        self.session.logger.info('model job started')

    def status(self):
        """Check the model job status"""
        self.session.logger.info(
            'checking model job status: model_id=%s job_type=%s',
            self.model_id,
            self._job_type,
        )
        return self.session.api_instance.model_job_controller_status(self.model_id, self._job_type)



# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 0.0.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import unittest

import curia.api.swagger_client
from curia.api.swagger_client.api.internal_api import InternalApi  # noqa: E501
from curia.api.swagger_client.rest import ApiException


class TestInternalApi(unittest.TestCase):
    """InternalApi unit test stubs"""

    def setUp(self):
        self.api = InternalApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_code_controller_get_upload_url(self):
        """Test case for code_controller_get_upload_url

        """
        pass

    def test_create_many_base_code_controller_code(self):
        """Test case for create_many_base_code_controller_code

        Create many Code  # noqa: E501
        """
        pass

    def test_create_many_base_feature_category_controller_feature_category(self):
        """Test case for create_many_base_feature_category_controller_feature_category

        Create many FeatureCategory  # noqa: E501
        """
        pass

    def test_create_many_base_feature_controller_feature(self):
        """Test case for create_many_base_feature_controller_feature

        Create many Feature  # noqa: E501
        """
        pass

    def test_create_many_base_feature_sub_category_controller_feature_sub_category(self):
        """Test case for create_many_base_feature_sub_category_controller_feature_sub_category

        Create many FeatureSubCategory  # noqa: E501
        """
        pass

    def test_create_one_base_code_controller_code(self):
        """Test case for create_one_base_code_controller_code

        Create one Code  # noqa: E501
        """
        pass

    def test_create_one_base_feature_category_controller_feature_category(self):
        """Test case for create_one_base_feature_category_controller_feature_category

        Create one FeatureCategory  # noqa: E501
        """
        pass

    def test_create_one_base_feature_controller_feature(self):
        """Test case for create_one_base_feature_controller_feature

        Create one Feature  # noqa: E501
        """
        pass

    def test_create_one_base_feature_sub_category_controller_feature_sub_category(self):
        """Test case for create_one_base_feature_sub_category_controller_feature_sub_category

        Create one FeatureSubCategory  # noqa: E501
        """
        pass

    def test_delete_one_base_code_controller_code(self):
        """Test case for delete_one_base_code_controller_code

        Delete one Code  # noqa: E501
        """
        pass

    def test_delete_one_base_feature_category_controller_feature_category(self):
        """Test case for delete_one_base_feature_category_controller_feature_category

        Delete one FeatureCategory  # noqa: E501
        """
        pass

    def test_delete_one_base_feature_controller_feature(self):
        """Test case for delete_one_base_feature_controller_feature

        Delete one Feature  # noqa: E501
        """
        pass

    def test_delete_one_base_feature_sub_category_controller_feature_sub_category(self):
        """Test case for delete_one_base_feature_sub_category_controller_feature_sub_category

        Delete one FeatureSubCategory  # noqa: E501
        """
        pass

    def test_get_many_base_code_controller_code(self):
        """Test case for get_many_base_code_controller_code

        Retrieve many Code  # noqa: E501
        """
        pass

    def test_get_many_base_feature_category_controller_feature_category(self):
        """Test case for get_many_base_feature_category_controller_feature_category

        Retrieve many FeatureCategory  # noqa: E501
        """
        pass

    def test_get_many_base_feature_controller_feature(self):
        """Test case for get_many_base_feature_controller_feature

        Retrieve many Feature  # noqa: E501
        """
        pass

    def test_get_many_base_feature_sub_category_controller_feature_sub_category(self):
        """Test case for get_many_base_feature_sub_category_controller_feature_sub_category

        Retrieve many FeatureSubCategory  # noqa: E501
        """
        pass

    def test_get_one_base_code_controller_code(self):
        """Test case for get_one_base_code_controller_code

        Retrieve one Code  # noqa: E501
        """
        pass

    def test_get_one_base_feature_category_controller_feature_category(self):
        """Test case for get_one_base_feature_category_controller_feature_category

        Retrieve one FeatureCategory  # noqa: E501
        """
        pass

    def test_get_one_base_feature_controller_feature(self):
        """Test case for get_one_base_feature_controller_feature

        Retrieve one Feature  # noqa: E501
        """
        pass

    def test_get_one_base_feature_sub_category_controller_feature_sub_category(self):
        """Test case for get_one_base_feature_sub_category_controller_feature_sub_category

        Retrieve one FeatureSubCategory  # noqa: E501
        """
        pass

    def test_health_controller_health(self):
        """Test case for health_controller_health

        """
        pass

    def test_replace_one_base_code_controller_code(self):
        """Test case for replace_one_base_code_controller_code

        Replace one Code  # noqa: E501
        """
        pass

    def test_replace_one_base_feature_category_controller_feature_category(self):
        """Test case for replace_one_base_feature_category_controller_feature_category

        Replace one FeatureCategory  # noqa: E501
        """
        pass

    def test_replace_one_base_feature_controller_feature(self):
        """Test case for replace_one_base_feature_controller_feature

        Replace one Feature  # noqa: E501
        """
        pass

    def test_replace_one_base_feature_sub_category_controller_feature_sub_category(self):
        """Test case for replace_one_base_feature_sub_category_controller_feature_sub_category

        Replace one FeatureSubCategory  # noqa: E501
        """
        pass

    def test_sns_message_controller_handle_message(self):
        """Test case for sns_message_controller_handle_message

        """
        pass

    def test_sns_message_controller_handle_message_0(self):
        """Test case for sns_message_controller_handle_message_0

        """
        pass

    def test_update_one_base_code_controller_code(self):
        """Test case for update_one_base_code_controller_code

        Update one Code  # noqa: E501
        """
        pass

    def test_update_one_base_feature_category_controller_feature_category(self):
        """Test case for update_one_base_feature_category_controller_feature_category

        Update one FeatureCategory  # noqa: E501
        """
        pass

    def test_update_one_base_feature_controller_feature(self):
        """Test case for update_one_base_feature_controller_feature

        Update one Feature  # noqa: E501
        """
        pass

    def test_update_one_base_feature_sub_category_controller_feature_sub_category(self):
        """Test case for update_one_base_feature_sub_category_controller_feature_sub_category

        Update one FeatureSubCategory  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



from logzero import logger
from pandas import DataFrame, Series
from validation_decorators import ValidateArgType
from validation_decorators.errors import raise_error

from curia.model import _Model
from curia.session import Session

type_validator = ValidateArgType(raise_error, logger=logger)


class RiskModel(_Model):
    """Create a Curia risk model"""

    @type_validator(session=Session, name=str)
    def __init__(
            self,
            session,
            name: str,
            description: str = None,
            project_id: str = None,
            environment_id: str = None,
    ):

        self.session = session
        self.model_id = None
        self.model_type = 'risk'

        super().__init__(
            session=self.session,
            name=name,
            description=description,
            project_id=project_id,
            environment_id=environment_id,
            model_type=self.model_type,
        )

    @type_validator(features=DataFrame, label=Series)
    def train(self, features: DataFrame, label: Series):
        """

        Parameters
        ----------
        features: DataFrame :
            
        label: Series :
            

        Returns
        -------

        """

        df = features.copy()
        df['label'] = label

        self.set_job_type('train')

        self.model_dataset_upload(df)

        self.start()

        # TODO - how to handle wait=True param (check status every n seconds)

    @type_validator(features=DataFrame)
    def predict(self, features: DataFrame):
        """

        Parameters
        ----------
        features: DataFrame :
            

        Returns
        -------

        """

        df = features.copy()

        self.set_job_type('predict')

        self.model_dataset_upload(df)

        self.start()

        # TODO - how to handle wait=True param (check status every n seconds)
